# -----------------------------------------------------------------------------
from dhenara.ai.types.shared.base import BaseEnum


class PromptMessageRoleEnum(BaseEnum):
    USER = "user"
    ASSISTANT = "assistant"
    SYSTEM = "system"
